import pandas as pd

import pystv
from pystv import RaceMetadata, RaceResult
from pystv import RoundResult as RR


def test_results_to_sankey():
    names = ["A", "B", "C"]
    race_result = RaceResult(
        RaceMetadata("race", 2, names),
        [
            RR([0, 2, 5, 3], [2], [], {2: {1: 1}}),
            RR([0, 3, 4, 3], [], [1], {1: {3: 3}}),
            RR([0, 0, 4, 6], [3], [], {}),
        ],
    )
    df_nodes, df_links = pystv.result_to_sankey_data(race_result)

    node_colors = ["rgba(0,0,0,0)"] + pystv.NODE_PALETTE[1:4]
    df_nodes_desired = pd.DataFrame(
        {
            "round": [0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3],
            "id": [0, 1, 2, 3, 0, 1, 2, 3, 0, 1, 2, 3, 0, 1, 2, 3],
            "label": ([""] + names) * 4,
            "color": node_colors * 4,
        }
    )
    df_links_desired = pd.DataFrame(
        {
            "round": [0, 0, 0, 0, 1, 1, 1, 2, 2],
            "source": [1, 2, 2, 3, 5, 6, 7, 10, 11],
            "target": [5, 5, 6, 7, 11, 10, 11, 14, 15],
            "value": [2, 1, 4, 3, 3, 4, 3, 4, 6],
            "color": [pystv.LINK_PALETTE[i] for i in [1, 2, 2, 3, 1, 2, 3, 2, 3]],
        }
    )

    print(df_links)
    pd.testing.assert_frame_equal(df_nodes, df_nodes_desired, check_like=True)
    pd.testing.assert_frame_equal(df_links, df_links_desired, check_like=True)

    # Just make sure this executes (not sure how to test figure generation)
    fig = pystv.create_sankey_fig(df_nodes, df_links)
    assert fig is not None
